@@ -2,11 +2,14 @@
 //! to enable fast bounding box spatial filtering.
 
 #[cfg(feature = "http")]
-use crate::http_client::BufferedHttpRangeClient;
+use crate::http_reader::from_http_err;
+use byteorder::{LittleEndian, ReadBytesExt, WriteBytesExt};
 use geozero::error::{GeozeroError, Result};
+#[cfg(feature = "http")]
+use http_range_client::BufferedHttpRangeClient;
 use std::cmp::Reverse;
 use std::collections::{BinaryHeap, HashMap};
-use std::io::{Read, Seek, SeekFrom, Write};
+use std::io::{Cursor, Read, Seek, SeekFrom, Write};
 use std::mem::size_of;
 use std::{cmp, f64, u64, usize};
 
@@ -14,12 +17,12 @@ use std::{cmp, f64, u64, usize};
 #[repr(C)]
 /// R-Tree node
 pub struct NodeItem {
-    min_x: f64, // double
-    min_y: f64, // double
-    max_x: f64, // double
-    max_y: f64, // double
+    pub min_x: f64,
+    pub min_y: f64,
+    pub max_x: f64,
+    pub max_y: f64,
     /// Byte offset in feature data section
-    offset: u64, // uint64_t
+    pub offset: u64,
 }
 
 impl NodeItem {
@@ -43,6 +46,29 @@ impl NodeItem {
         }
     }
 
+    pub fn from_reader<R: Read + ?Sized>(rdr: &mut R) -> Result<Self> {
+        Ok(NodeItem {
+            min_x: rdr.read_f64::<LittleEndian>()?,
+            min_y: rdr.read_f64::<LittleEndian>()?,
+            max_x: rdr.read_f64::<LittleEndian>()?,
+            max_y: rdr.read_f64::<LittleEndian>()?,
+            offset: rdr.read_u64::<LittleEndian>()?,
+        })
+    }
+
+    fn from_bytes(raw: &[u8]) -> Result<Self> {
+        Self::from_reader(&mut Cursor::new(raw))
+    }
+
+    pub fn write<W: Write>(&self, wtr: &mut W) -> std::io::Result<()> {
+        wtr.write_f64::<LittleEndian>(self.min_x)?;
+        wtr.write_f64::<LittleEndian>(self.min_y)?;
+        wtr.write_f64::<LittleEndian>(self.max_x)?;
+        wtr.write_f64::<LittleEndian>(self.max_y)?;
+        wtr.write_u64::<LittleEndian>(self.offset)?;
+        Ok(())
+    }
+
     pub fn width(&self) -> f64 {
         self.max_x - self.min_x
     }
@@ -56,7 +82,7 @@ impl NodeItem {
         a
     }
 
-    fn expand(&mut self, r: &NodeItem) {
+    pub fn expand(&mut self, r: &NodeItem) {
         if r.min_x < self.min_x {
             self.min_x = r.min_x;
         }
@@ -71,6 +97,21 @@ impl NodeItem {
         }
     }
 
+    pub fn expand_xy(&mut self, x: f64, y: f64) {
+        if x < self.min_x {
+            self.min_x = x;
+        }
+        if y < self.min_y {
+            self.min_y = y;
+        }
+        if x > self.max_x {
+            self.max_x = x;
+        }
+        if y > self.max_y {
+            self.max_y = y;
+        }
+    }
+
     pub fn intersects(&self, r: &NodeItem) -> bool {
         if self.max_x < r.min_x {
             return false;
@@ -86,25 +127,13 @@ impl NodeItem {
         }
         true
     }
-
-    // std::vector<double> NodeItem::toVector()
-    // {
-    //     return std::vector<double> { min_x, min_y, max_x, max_y };
-    // }
 }
 
 /// Read full capacity of vec from data stream
-fn read_node_vec(node_items: &mut Vec<NodeItem>, data: &mut dyn Read) -> Result<()> {
-    let num_nodes = node_items.capacity();
-    let buf = unsafe {
-        std::slice::from_raw_parts_mut(
-            node_items.as_mut_ptr() as *mut u8,
-            num_nodes * size_of::<NodeItem>(),
-        )
-    };
-    data.read_exact(buf)?;
-    unsafe {
-        node_items.set_len(num_nodes);
+fn read_node_vec<R: Read + ?Sized>(node_items: &mut Vec<NodeItem>, data: &mut R) -> Result<()> {
+    node_items.clear();
+    for _ in 0..node_items.capacity() {
+        node_items.push(NodeItem::from_reader(data)?);
     }
     Ok(())
 }
@@ -135,13 +164,16 @@ async fn read_http_node_items(
 ) -> Result<Vec<NodeItem>> {
     let begin = base + node_index * size_of::<NodeItem>();
     let len = length * size_of::<NodeItem>();
-    let bytes = client.get_range(begin, len, min_req_size).await?;
+    let bytes = client
+        .get_range(begin, len, min_req_size)
+        .await
+        .map_err(from_http_err)?;
 
     let mut node_items = Vec::with_capacity(length);
-    let buf = unsafe { std::slice::from_raw_parts_mut(node_items.as_mut_ptr() as *mut u8, len) };
-    buf.clone_from_slice(&bytes);
-    unsafe {
-        node_items.set_len(length);
+    for i in 0..length {
+        node_items.push(NodeItem::from_bytes(
+            &bytes[i * size_of::<NodeItem>()..(i + 1) * size_of::<NodeItem>()],
+        )?);
     }
     Ok(node_items)
 }
@@ -224,28 +256,14 @@ fn hilbert_bbox(r: &NodeItem, hilbert_max: u32, extent: &NodeItem) -> u32 {
     hilbert(x, y)
 }
 
-pub fn hilbert_sort(items: &mut Vec<NodeItem>) {
-    let extent = calc_extent(items);
+pub fn hilbert_sort(items: &mut Vec<NodeItem>, extent: &NodeItem) {
     items.sort_by(|a, b| {
-        let ha = hilbert_bbox(a, HILBERT_MAX, &extent);
-        let hb = hilbert_bbox(b, HILBERT_MAX, &extent);
+        let ha = hilbert_bbox(a, HILBERT_MAX, extent);
+        let hb = hilbert_bbox(b, HILBERT_MAX, extent);
         hb.partial_cmp(&ha).unwrap() // ha > hb
     });
 }
 
-// void hilbert_sort_shared_ptr(std::vector<std::shared_ptr<Item>> &items)
-// {
-//     NodeItem extent = std::accumulate(items.begin(), items.end(), NodeItem::create(0), [] (NodeItem a, std::shared_ptr<Item> b) {
-//         a.expand(b->node_item);
-//         return a;
-//     });
-//     std::sort(items.begin(), items.end(), [&extent] (std::shared_ptr<Item> a, std::shared_ptr<Item> b) {
-//         uint32_t ha = hilbert(a->node_item, hilbert_max, extent);
-//         uint32_t hb = hilbert(b->node_item, hilbert_max, extent);
-//         return ha > hb;
-//     });
-// }
-
 pub fn calc_extent(nodes: &Vec<NodeItem>) -> NodeItem {
     nodes.iter().fold(NodeItem::create(0), |mut a, b| {
         a.expand(b);
@@ -253,15 +271,6 @@ pub fn calc_extent(nodes: &Vec<NodeItem>) -> NodeItem {
     })
 }
 
-// NodeItem calc_extent_shared_ptr(const std::vector<std::shared_ptr<Item>> &items)
-// {
-//     NodeItem extent = std::accumulate(items.begin(), items.end(), NodeItem::create(0), [] (NodeItem a, std::shared_ptr<Item> b) {
-//         a.expand(b->node_item);
-//         return a;
-//     });
-//     return extent;
-// }
-
 /// Packed Hilbert R-Tree
 pub struct PackedRTree {
     extent: NodeItem,
@@ -366,11 +375,11 @@ impl PackedRTree {
         for i in 0..self.num_nodes {
             let bytes = client
                 .get_range(pos, size_of::<NodeItem>(), min_req_size)
-                .await?;
-            let p = bytes.as_ptr() as *const [u8; size_of::<NodeItem>()];
-            let n: NodeItem = unsafe { std::mem::transmute(*p) };
-            self.node_items[i] = n.clone();
+                .await
+                .map_err(from_http_err)?;
+            let n = NodeItem::from_bytes(bytes)?;
             self.extent.expand(&n);
+            self.node_items[i] = n;
             pos += size_of::<NodeItem>();
         }
         Ok(())
@@ -661,14 +670,12 @@ impl PackedRTree {
         num_nodes * size_of::<NodeItem>()
     }
 
-    pub fn stream_write(&self, out: &mut dyn Write) -> std::io::Result<()> {
-        let buf: &[u8] = unsafe {
-            std::slice::from_raw_parts(
-                self.node_items.as_ptr() as *const u8,
-                self.node_items.len() * size_of::<NodeItem>(),
-            )
-        };
-        out.write_all(buf)
+    /// Write all index nodes
+    pub fn stream_write<W: Write>(&self, out: &mut W) -> std::io::Result<()> {
+        for item in &self.node_items {
+            item.write(out)?;
+        }
+        Ok(())
     }
 
     pub fn extent(&self) -> NodeItem {
@@ -692,7 +699,8 @@ mod inspect {
                     let _ =
                         processor.property(0, "levelno", &ColumnValue::ULong(levelno as u64))?;
                     let _ = processor.property(1, "pos", &ColumnValue::ULong(pos as u64))?;
-                    let _ = processor.property(2, "offset", &ColumnValue::ULong(node.offset))?;
+                    let _ =
+                        processor.property(2, "offset", &ColumnValue::ULong(node.offset as u64))?;
                     processor.properties_end()?;
                     processor.geometry_begin()?;
                     processor.polygon_begin(true, 1, 0)?;
@@ -723,11 +731,11 @@ fn tree_2items() -> Result<()> {
     assert_eq!(extent, NodeItem::new(0.0, 0.0, 3.0, 3.0));
     assert!(nodes[0].intersects(&NodeItem::new(0.0, 0.0, 1.0, 1.0)));
     assert!(nodes[1].intersects(&NodeItem::new(2.0, 2.0, 3.0, 3.0)));
-    hilbert_sort(&mut nodes);
+    hilbert_sort(&mut nodes, &extent);
     let mut offset = 0;
     for mut node in &mut nodes {
-        node.offset = offset as u64;
-        offset += size_of::<NodeItem>();
+        node.offset = offset;
+        offset += size_of::<NodeItem>() as u64;
     }
     assert!(nodes[1].intersects(&NodeItem::new(0.0, 0.0, 1.0, 1.0)));
     assert!(nodes[0].intersects(&NodeItem::new(2.0, 2.0, 3.0, 3.0)));
@@ -761,11 +769,11 @@ fn tree_19items_roundtrip_stream_search() -> Result<()> {
     nodes.push(NodeItem::new(10010.0, 10010.0, 10110.0, 10110.0));
     nodes.push(NodeItem::new(10010.0, 10010.0, 10110.0, 10110.0));
     let extent = calc_extent(&nodes);
-    hilbert_sort(&mut nodes);
+    hilbert_sort(&mut nodes, &extent);
     let mut offset = 0;
     for mut node in &mut nodes {
-        node.offset = offset as u64;
-        offset += size_of::<NodeItem>();
+        node.offset = offset;
+        offset += size_of::<NodeItem>() as u64;
     }
     let tree = PackedRTree::build(&nodes, &extent, PackedRTree::DEFAULT_NODE_SIZE)?;
     let list = tree.search(102.0, 102.0, 103.0, 103.0)?;
@@ -823,7 +831,7 @@ fn tree_100_000_items_in_denmark() -> Result<()> {
     }
 
     let extent = calc_extent(&nodes);
-    hilbert_sort(&mut nodes);
+    hilbert_sort(&mut nodes, &extent);
     let tree = PackedRTree::build(&nodes, &extent, PackedRTree::DEFAULT_NODE_SIZE)?;
     let list = tree.search(690407.0, 6063692.0, 811682.0, 6176467.0)?;
 
@@ -866,8 +874,8 @@ fn tree_processing() -> Result<()> {
     let extent = calc_extent(&nodes);
     let mut offset = 0;
     for mut node in &mut nodes {
-        node.offset = offset as u64;
-        offset += size_of::<NodeItem>();
+        node.offset = offset;
+        offset += size_of::<NodeItem>() as u64;
     }
     let tree = PackedRTree::build(&nodes, &extent, PackedRTree::DEFAULT_NODE_SIZE)?;
     let mut fout = BufWriter::new(tempfile()?);
